@@ -3,10 +3,10 @@ __author__ = ('Aitor Blanco Miguez (aitor.blancomiguez@unitn.it), '
               'Francesco Asnicar (f.asnicar@unitn.it), '
               'Moreno Zolfo (moreno.zolfo@unitn.it), '
               'Francesco Beghini (francesco.beghini@unitn.it)')
-__version__ = '3.0'
-__date__ = '1 Sep 2020'
+__version__ = '4.0.1'
+__date__ = '24 Aug 2022'
 
-import os, sys, re, shutil
+import os, sys, re, shutil, tempfile
 import subprocess as sb
 try:
     from .util_fun import info, error
@@ -135,7 +135,7 @@ Executes PhyloPhlAn
 :param mutation_rates: whether get  the mutation rates for the markers
 :param nproc: the number of threads to run phylophlan
 """
-def execute_phylophlan(samples_markers_dir, conf_file, min_entries, tmp_dir, output_dir, 
+def execute_phylophlan(samples_markers_dir, conf_file, min_entries, min_markers, tmp_dir, output_dir, 
     clade, phylogeny_conf, mutation_rates, nprocs):
     accuracy = " --"+phylogeny_conf
 
@@ -143,10 +143,11 @@ def execute_phylophlan(samples_markers_dir, conf_file, min_entries, tmp_dir, out
         accuracy = accuracy + " --mutation_rates"
     params = {
         "program_name" : "phylophlan",
-        "params" : "-d "+clade+" --data_folder "+tmp_dir+
+        "params" : "-d "+clade[:30]+" --data_folder "+tmp_dir+
             " --databases_folder "+tmp_dir+" -t n -f "+conf_file+
             " --diversity low"+accuracy+" --genome_extension fna"+
-            " --force_nucleotides --min_num_entries "+str(min_entries),
+            " --force_nucleotides --min_num_entries "+str(min_entries)+
+            " --convert_N2gap --min_num_markers "+str(min_markers),
         "input" : "-i",
         "output_path" : "--output_folder",
         "output" : "-o",
@@ -244,7 +245,9 @@ Generates a FASTA file with the markers form a MetaPhlAn database
 :param output_dir: the output directory
 """
 def generate_markers_fasta(database, output_dir):
-    db_markers_faa = output_dir+"db_markers.fna"
+    
+    file_out, db_markers_faa = tempfile.mkstemp(dir=output_dir,prefix="db_markers_temp_",suffix=".fna")
+    os.close(file_out)
     bowtie_database, _ = os.path.splitext(database)
     params = {
         "program_name" : "bowtie2-inspect",
